<?php
// 2-Sterne-Kabinen
class LowCabin extends BaseCabin implements ItemIsTradeable, ConstructableShipPart {
	// Konstruktor
	private function __construct () {
		// Eltern-Konstruktor aufrufen
		parent::constructor(__CLASS__);

		// Debug message
		if (((defined('DEBUG_CABIN')) || (defined('DEBUG_ALL'))) && (defined('DEBUG_CONSTRUCT'))) $this->getDebugInstance()->output(sprintf("[%s:%d] Konstruktor erreicht.<br />\n",
			__CLASS__,
			__LINE__
		));

		// Beschreibung setzen
		$this->setPartDescr("2Star-Class-Kabine");

		// Unique-ID erzeugen
		$this->createUniqueID();

		// Clean up a little
		$this->removeSystemArray();
	}

	// 2-Sterne-Klasse erstellen
	public static function createLowCabin ($numLuxury, $numRooms, $numBeds, $dim) {
		// Instanz holen
		$lowInstance = new LowCabin();

		// Debug message
		if ((defined('DEBUG_CABIN')) || (defined('DEBUG_ALL'))) $lowInstance->getDebugInstance()->output(sprintf("[%s:%d] Eine 2-Sterne-Kabine wird konstruiert...<br />\n",
			__CLASS__,
			__LINE__
		));

		// Abmasse extrahieren
		$lowInstance->extractDimensions($dim);

		// Den Rest auch setzen
		$lowInstance->setNumCabin($numLuxury);
		$lowInstance->setNumRooms($numRooms);
		$lowInstance->setNumBeds($numBeds);

		// Nicht noetig!
		$lowInstance->removePartInstance();

		// Instanz zurueckgeben
		return $lowInstance;
	}

	// Loesch-Methode fuer Anzahl Betten
	public function removeNumBeds() {
		if ((defined('DEBUG_CABIN')) || (defined('DEBUG_ALL'))) $this->getDebugInstance()->output(sprintf("[%s:%d] Anzahl Betten gel&ouml;scht.<br />\n",
			__CLASS__,
			__LINE__
		));
		unset($this->numBeds);
		parent::removeNumBeds();
	}

	// Loesch-Methode fuer Anzahl Kabinen
	public function removeNumCabin() {
		if ((defined('DEBUG_CABIN')) || (defined('DEBUG_ALL'))) $this->getDebugInstance()->output(sprintf("[%s:%d] Anzahl Kabinen gel&ouml;scht.<br />\n",
			__CLASS__,
			__LINE__
		));
		unset($this->numCabin);
		parent::removeNumCabin();
	}

	// Loesch-Methode fuer Anzahl Raeume
	public function removeNumRooms() {
		if ((defined('DEBUG_CABIN')) || (defined('DEBUG_ALL'))) $this->getDebugInstance()->output(sprintf("[%s:%d] Anzahl R&auml;ume gel&ouml;scht.<br />\n",
			__CLASS__,
			__LINE__
		));
		unset($this->numRooms);
		parent::removeNumRooms();
	}

	// Overwritten method for tradeable items
	public function isTradeable () {
		return true;
	}

	/**
	 * Limits this object with an ObjectLimits instance
	 */
	public function limitObject (ObjectLimits $limitInstance) {
		ApplicationEntryPoint::app_die("".__METHOD__." reached! Stub!");
	}
}

// [EOF]
?>
