<?php
// Premier-Kabinen
class PremierCabin extends BaseCabin implements ItemIsTradeable, ConstructableShipPart {
	// Konstruktor
	private function __construct () {
		// Eltern-Konstruktor aufrufen
		parent::constructor(__CLASS__);

		// Debug message
		if (((defined('DEBUG_CABIN')) || (defined('DEBUG_ALL'))) && (defined('DEBUG_CONSTRUCT'))) $this->getDebugInstance()->output(sprintf("[PremierCabin:] Konstruktor erreicht.<br />\n",
			__CLASS__,
			__LINE__
		));

		// Beschreibung setzen
		$this->setPartDescr("Premier-Class-Kabine");

		// Unique-ID erzeugen
		$this->createUniqueID();

		// Clean up a little
		$this->removeSystemArray();
	}

	// Premier-Kabine erstellen
	public static function createPremierCabin ($numLuxury, $numRooms, $numBeds, $dim) {
		// Instanz holen
		$premierInstance = new PremierCabin();

		// Debug message
		if ((defined('DEBUG_CABIN')) || (defined('DEBUG_ALL'))) $premierInstance->getDebugInstance()->output("[PremierCabin:] Eine Premier-Kabine wird konstruiert...<br />\n");

		// Abmasse extrahieren
		$premierInstance->extractDimensions($dim);

		// Den Rest auch setzen
		$premierInstance->setNumCabin($numLuxury);
		$premierInstance->setNumRooms($numRooms);
		$premierInstance->setNumBeds($numBeds);

		// Nicht noetig!
		$premierInstance->removePartInstance();

		// Instanz zurueckgeben
		return $premierInstance;
	}

	// Loesch-Methode fuer Anzahl Betten
	public function removeNumBeds() {
		if ((defined('DEBUG_CABIN')) || (defined('DEBUG_ALL'))) $this->getDebugInstance()->output(sprintf("[%s:%d] Anzahl Betten gel&ouml;scht.<br />\n",
			__CLASS__,
			__LINE__
		));
		unset($this->numBeds);
		parent::removeNumBeds();
	}

	// Loesch-Methode fuer Anzahl Kabinen
	public function removeNumCabin() {
		if ((defined('DEBUG_CABIN')) || (defined('DEBUG_ALL'))) $this->getDebugInstance()->output(sprintf("[%s:%d] Anzahl Kabinen gel&ouml;scht.<br />\n",
			__CLASS__,
			__LINE__
		));
		unset($this->numCabin);
		parent::removeNumCabin();
	}

	// Loesch-Methode fuer Anzahl Raeume
	public function removeNumRooms() {
		if ((defined('DEBUG_CABIN')) || (defined('DEBUG_ALL'))) $this->getDebugInstance()->output(sprintf("[%s:%d] Anzahl R&auml;ume gel&ouml;scht.<br />\n",
			__CLASS__,
			__LINE__
		));
		unset($this->numRooms);
		parent::removeNumRooms();
	}

	// Overwritten method for tradeable items
	public function isTradeable () {
		return true;
	}

	/**
	 * Limits this object with an ObjectLimits instance
	 */
	public function limitObject (ObjectLimits $limitInstance) {
		ApplicationEntryPoint::app_die("".__METHOD__." reached! Stub!");
	}
}

// [EOF]
?>
